/*#################################################################
# Copyright (c) 1991 AT&T Bell Laboratories, All Rights Reserved   
# Published in ``A C++ Primer''  by Stanley Lippman, Addison-Wesley
#################################################################*/

#include <iostream.h>
#include <string.h>
#include "Text.h"

Text::Text()
{
        len = 0;
        str = 0;
}

Text::Text( char *s ) 
{
        if ( !s ) { 
                len = 0; 
                str = 0;
        }
        else {
                len = (int) strlen( s );
                str = new char[ len + 1 ];
                strcpy( str, s );
        }
}

Text::Text( const Text& s )
{
        len = s.len;
        if ( (str = s.str) == 0) 
              return; 

        str = new char[ len + 1 ];
        strcpy( str, s.str );
}

Text& Text::operator=( const Text& s )
{
    if (this == &s) return *this;
    delete str;

    len = s.len;
    if ( (str = s.str) == 0 )  
          return *this; 

    str = new char[ len + 1 ];
    strcpy( str, s.str );
    return *this;
}

int Text::operator==(const Text &s) 
{
    if (len != s.len) return 0;
    return (strcmp(str,s.str)==0); 
}

char&
Text::operator[]( int elem )
{
    return str[ elem ];
}

#include <iomanip.h>
istream& operator>>( istream &io, Text &s )
{
    const int Text_size = 1024;
    char inBuf[ Text_size ]; 
    io >> setw( Text_size-1 ) >> inBuf;
    s = inBuf; // Text::operator=(char*);
    return io;
}

ostream& operator<<( ostream& os, Text& s )
{
    if (!s) return os << "<empty>";
    return os << s.str;
}

Text& Text::operator=( const char *s )
{
    if (s == 0) { len = 0; str = 0; return *this; }
    len = (int) strlen( s );
    delete str;
    str = new char[ len + 1 ];
    strcpy( str, s );
    return *this;
}

// return substring starting at pos of length cnt
Text Text::operator()(int pos, int cnt) 
{
    assert ( pos >= 0 && pos < len && cnt >= 0 );

    if ( cnt + pos - 1 > len ) {
        cerr << "warning: Text::operator( " << pos 
                 << ", " << cnt << " ) substring truncation "
                 << str << endl;
        cnt = len - pos + 1;
    }

    char *ps = new char[ cnt+1 ];
    for ( int i = pos, j = 0; j < cnt; ++i, ++j ) 
          ps[ j ] = str[ i ];
    ps[cnt] = '\0';

    return Text( ps );
}

// return substring from pos to the first occurence of ch
Text Text::operator()(int pos, char ch) 
{
  assert ( pos >= 0 && pos < len);

  char *location = strchr(&str[pos],ch);

  if (location == NULL) return NULL;

  int length = (int) (location - &str[pos]); assert (length < len);

  char *ps = new char[ length+1 ];
  for ( int i = pos, j = 0; j < length; ++i, ++j ) 
    ps[ j ] = str[ i ];
  ps[length] = '\0';

  return Text( ps );
}

// return substring from beginning to the last occurence of ch
Text Text::operator()(char ch) 
{
  char *location = strrchr(str,ch);

  if (location == NULL) return NULL;

  int length = (int) (location - str); assert (length < len);

  char *ps = new char[ length+1 ];
  for ( int j = 0; j < length; ++j ) 
    ps[ j ] = str[ j ];
  ps[length] = '\0';

  return Text( ps );
}

Text Text::fromLastCharToEnd(char ch)
{
  char *location = strrchr(str,ch);

  if (location == NULL) return NULL;

  int length = (int) strlen(location); assert (length < len);

  char *ps = new char[ length + 1];
  for ( int j = 0; j < length; ++j ) 
    ps[ j ] = location[ j+1 ];
  ps[length] = '\0';

  return Text( ps );

}

// return substring starting at pos
Text Text::operator()(int pos) 
{
    assert ( pos >= 0 && pos < len);
   
    int cnt = len - pos + 1;
    char *ps = new char[ cnt+1 ];
    for ( int i = pos, j = 0; j < cnt; ++i, ++j ) 
          ps[ j ] = str[ i ];
    ps[cnt] = '\0';

    return Text( ps );
}

Text&
Text::operator+=( const Text &s )
{
    len += s.len;
    if ( !len )
         return *this;
        
    char *p = new char[len+1];
    strcpy(p,str);
    strcat(p,s.str);
    delete str;
    str=p;
    return *this;
}

Text&
Text::operator+=( char ch )
{
    len += 1;
    if ( !len )
         return *this;
        
    char *p = new char[len+1];
    strcpy(p,str);
    strcat(p,&ch);
    delete str;
    str=p;
    return *this;
}

Text
operator+(const Text &s1, const Text &s2 )
{
    Text result = s1;
    result += s2; 
    return result;
}

int Text::operator<(Text& s) { 
        if (s.len == 0) return 0;
        if (len == 0) return 1;
        return(strcmp(str,s.str)<0) ? 1: 0; 
}

int Text::operator>(Text& s) { 
        if (len == 0) return 0;
        if (s.len == 0) return 1;
        return(strcmp(str,s.str)>0) ? 1: 0; 
}

